public class Driver {
    
    final static int WAMRUP_ROUNDS = 100000;
    final static int BENCH_ROUNDS = 100000;
    final static int BENCH_ITERATIONS = 30;
    final static int ADDITIONAL_ASSOCIATIONS = 10000;
    
    public static void main(String[] args) {
        Bit b1 = new Bit();
        Bit b2 = new Bit();
        Bit b3 = new Bit();

        associate(b1, b2);
        associate(b2, b3);
        
        /*
         * Run correctness checks
         */
        
        b1.set();
        if (!(b1.get() && b2.get() && b3.get())) {
            System.out.println(b1);
            System.out.println(b2);
            System.out.println(b3);
            throw new RuntimeException();
        }
            
        b2.clear();
        if (!(b1.get() && !b2.get() && !b3.get())) {
            throw new RuntimeException();
        }
        
        b3.set();
        if (!(b1.get() && !b2.get() && b3.get())) {
            throw new RuntimeException();
        }
        
        System.out.println("Tests passed. Warmup and benchmark follow.");

        /*
         * warmup
         */

        for(int i=0;i<WAMRUP_ROUNDS;i++) {
            b1.set();
            b2.set();
            b3.set();
            b1.clear();
            b2.clear();
            b3.clear();
        }
        
        /*
         * measuring overhead of relational advice execution
         */

        long start, acc;
        
        acc=0;
        for(int i=0;i<BENCH_ITERATIONS;i++) {
            start = System.currentTimeMillis();
            for(int r=0;r<BENCH_ROUNDS;r++) {
                b1.set();
                b2.set();
                b3.set();
                b1.clear();
                b2.clear();
                b3.clear();
            }
            long diff = System.currentTimeMillis()-start;
            acc += diff;
            System.out.println(diff);
            System.gc();
        }
        System.out.println("mean: "+(acc/BENCH_ITERATIONS));

        /*
         * Create additional associations.
         */
        
        for(int i=0;i<ADDITIONAL_ASSOCIATIONS;i++) {
            associate(new Bit(),b2);            
        }
        
        /*
         * measuring overhead of relational advice execution
         */

        acc=0;
        for(int i=0;i<BENCH_ITERATIONS;i++) {
            start = System.currentTimeMillis();
            for(int r=0;r<BENCH_ROUNDS;r++) {
                b1.set();
                b2.set();
                b3.set();
                b1.clear();
                b2.clear();
                b3.clear();
            }
            long diff = System.currentTimeMillis()-start;
            acc += diff;
            System.out.println(diff);
            System.gc();
        }
        System.out.println("mean: "+(acc/BENCH_ITERATIONS));
        
    }

    public static void associate(Bit b1, Bit b2) {
        // to be advised by the aspect implementations
    }
}
