public class Driver {
    
    final static int ITERATIONS = 30;
    final static int ROUNDS = 50000;
    final static int AUX_OBJECTS = 3000;    
    
    static Programmer p1 = new Programmer("p1");
    static Programmer p2 = new Programmer("p2");
    static Programmer p3 = new Programmer("p3");
    
    static Reviewer r1 = new Reviewer("r1");
    static Reviewer r2 = new Reviewer("r2");
    
    static Documenter d = new Documenter("d ");
    
    static Software s1 = new Software("s1");
    static Software s2 = new Software("s2");
    static Software s3 = new Software("s3");

    public static void main(String[] args) {

        System.out.println("Warming up...");
        // warmup
        doAssociate(p1,r1,d,s1);
        doAssociate(p2,r1,d,s1);
        for(int r=0;r<5*ROUNDS;r++) {
            p1.program(s1);
        }
        doRelease(p1,r1,d,s1);
        doRelease(p2,r1,d,s1);
        
        //actual run
        System.out.println("Warmup done.");

        for(int i=0;i<ITERATIONS;i++) {
            oneRound();
        }
    }
    
    static void oneRound() {
        p1 = new Programmer("p1");
        p2 = new Programmer("p2");
        p3 = new Programmer("p3");
        
        r1 = new Reviewer("r1");
        r2 = new Reviewer("r2");
        
        d = new Documenter("d ");
        
        s1 = new Software("s1");
        s2 = new Software("s2");
        s3 = new Software("s3");

        // Programmer p1 programs on his own private software s3.
        for(int r=0;r<ROUNDS;r++) {
            p1.program(s3);
        }
        
        // Programmer p2 does work on Software s1.
        for(int r=0;r<ROUNDS;r++) {
            p2.program(s1);
        }
        
        // Reviewer r2 helps with the review of s1.
        for(int r=0;r<ROUNDS;r++) {
            r2.review(s1);
        }
        
        // Programmer p1 programs on s1, still on its own...
        for(int r=0;r<ROUNDS;r++) {
            p1.program(s1);
        }

        //do some associations
        
        /*
         * Programmers p1 and p2 work together on Software s1,
         * which is reviewed by Reviewer r1 and documented by Documenter d. 
         */
        doAssociate(p1,r1,d,s1);
        doAssociate(p2,r1,d,s1);

        /*
         * Programmers p1 and p3 work together on Software s2,
         * which is reviewed by Reviewer r2 and documented by Documenter d. 
         */
        doAssociate(p1,r2,d,s2);
        doAssociate(p3,r2,d,s2);

        long bef = System.currentTimeMillis();

        //associate some other pieces of software to measure load on GC
        
        for(int r=0;r<AUX_OBJECTS;r++) {
            doAssociate(p1,r2,d,new Software("temp"));
        }

        //take time
        System.out.println("associate: "+(System.currentTimeMillis() - bef));

        //first GC: garbage collect newly-created "Software" objects if possible 
        System.gc();

        // Programmer p1 programs on s1
        //this will discard all disjuncts with weak references
        for(int r=0;r<ROUNDS;r++) {
            p1.program(s1);
        }

        //second GC: garbage collect obsolete disjuncts 
        System.gc();

        bef = System.currentTimeMillis();
        
        // Programmer p1 programs on s1
        for(int r=0;r<ROUNDS;r++) {
            p1.program(s1);
        }

        // Programmer p1 programs on his own private software s3.
        for(int r=0;r<ROUNDS;r++) {
            p1.program(s3);
        }
        
        // Programmer p2 does work on Software s1.
        for(int r=0;r<ROUNDS;r++) {
            p2.program(s1);
        }
        
        // Reviewer r2 helps with the review of s1.
        for(int r=0;r<ROUNDS;r++) {
            r2.review(s1);
        }
        
        //s1 is completed (re-instantiate value for next round)        
        s1 = null;
        
        // Programmer p1 programs on s2
        for(int r=0;r<ROUNDS;r++) {
            p1.program(s2);
        }
        
        //take time
        System.out.println("action:    "+(System.currentTimeMillis() - bef));
    }
    
    static void doAssociate(Programmer p, Reviewer r, Documenter d, Software s) {
        Working.associate(p,r,d,s);
    }
    
    static void doRelease(Programmer p, Reviewer r, Documenter d, Software s) {
        Working.release(p,r,d,s);
    }
}